__name__ = "aiogoogle"
__about__ = "Async Google API client"
__description__ = __about__
__url__ = "https://github.com/omarryhan/aiogoogle"
__version_info__ = ("5", "2", "0")
__version__ = ".".join(__version_info__)
__author__ = "Omar Ryhan"
__author_email__ = "omarryhan@gmail.com"
__maintainer__ = "Omar Ryhan"
__license__ = "MIT"
__copyright__ = "(c) 2018 by Omar Ryhan"
